@@ -1,4 +1,4 @@
-# (c) Copyright [2018-2020] Micro Focus or one of its affiliates.
+# (c) Copyright [2018-2021] Micro Focus or one of its affiliates.
 # Licensed under the Apache License, Version 2.0 (the "License");
 # You may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -51,6 +51,10 @@
 # Standard Python Modules
 import math
 from collections.abc import Iterable
+from typing import Union
+
+# Other Modules
+import numpy as np
 
 # VerticaPy Modules
 from verticapy import *
@@ -62,9 +66,143 @@ from verticapy.toolbox import *
 #
 # Regression
 #
+# ---#
+def aic_bic(
+    y_true: str,
+    y_score: str,
+    input_relation: Union[str, vDataFrame],
+    cursor=None,
+    k: int = 1,
+):
+    """
+---------------------------------------------------------------------------
+Computes the AIC (Akaike’s Information Criterion) & BIC (Bayesian Information 
+Criterion).
+
+Parameters
+----------
+y_true: str
+    Response column.
+y_score: str
+    Prediction.
+input_relation: str/vDataFrame
+    Relation to use to do the scoring. The relation can be a view or a table
+    or even a customized relation. For example, you could write:
+    "(SELECT ... FROM ...) x" as long as an alias is given at the end of the
+    relation.
+cursor: DBcursor, optional
+    Vertica database cursor.
+k: int, optional
+    Number of predictors.
+
+Returns
+-------
+tuple of floats
+    (AIC, BIC)
+    """
+    check_types(
+        [
+            ("y_true", y_true, [str],),
+            ("y_score", y_score, [str],),
+            ("input_relation", input_relation, [str, vDataFrame],),
+            ("k", k, [int],),
+        ]
+    )
+    cursor, conn, input_relation = check_cursor(cursor, input_relation)
+    query = "SELECT SUM(POWER({} - {}, 2)), COUNT(*) FROM {}".format(
+        y_true, y_score, input_relation
+    )
+    executeSQL(cursor, query, "Computing the RSS Score.")
+    rss, n = cursor.fetchone()
+    if rss > 0:
+        result = (
+            n * math.log(rss / n) + 2 * (k + 1) + (2 * (k + 1) ** 2 + 2 * (k + 1)) / (n - k - 2),
+            n * math.log(rss / n) + (k + 1) * math.log(n),
+        )
+    else:
+        result = -float("inf"), -float("inf")
+    if conn:
+        conn.close()
+    return result
+
+
+# ---#
+def anova_table(
+    y_true: str,
+    y_score: str,
+    input_relation: Union[str, vDataFrame],
+    k: int = 1,
+    cursor=None,
+):
+    """
+---------------------------------------------------------------------------
+Computes the Anova Table.
+
+Parameters
+----------
+y_true: str
+    Response column.
+y_score: str
+    Prediction.
+input_relation: str/vDataFrame
+    Relation to use to do the scoring. The relation can be a view or a table
+    or even a customized relation. For example, you could write:
+    "(SELECT ... FROM ...) x" as long as an alias is given at the end of the
+    relation.
+k: int, optional
+    Number of predictors.
+cursor: DBcursor, optional
+    Vertica database cursor.
+
+Returns
+-------
+tablesample
+    An object containing the result. For more information, see
+    utilities.tablesample.
+    """
+    check_types(
+        [
+            ("y_true", y_true, [str],),
+            ("y_score", y_score, [str],),
+            ("input_relation", input_relation, [str, vDataFrame],),
+            ("k", k, [int],),
+        ]
+    )
+    cursor, conn, input_relation = check_cursor(cursor, input_relation)
+    query = "SELECT COUNT(*), AVG({}) FROM {}".format(y_true, input_relation)
+    executeSQL(cursor, query, "Computing n and the average of y.")
+    n, avg = cursor.fetchone()[0:2]
+    query = "SELECT SUM(POWER({} - {}, 2)), SUM(POWER({} - {}, 2)), SUM(POWER({} - {}, 2)) FROM {}".format(
+        y_score, avg, y_true, y_score, y_true, avg, input_relation
+    )
+    executeSQL(cursor, query, "Computing SSR, SSE, SST.")
+    SSR, SSE, SST = cursor.fetchone()[0:3]
+    dfr, dfe, dft = k, n - 1 - k, n - 1
+    MSR, MSE = SSR / dfr, SSE / dfe
+    if MSE == 0:
+        F = float("inf")
+    else:
+        F = MSR / MSE
+    from scipy.stats import f
+
+    pvalue = f.sf(F, k, n)
+    if conn:
+        conn.close()
+    return tablesample(
+        {
+            "index": ["Regression", "Residual", "Total"],
+            "Df": [dfr, dfe, dft],
+            "SS": [SSR, SSE, SST],
+            "MS": [MSR, MSE, ""],
+            "F": [F, "", ""],
+            "p_value": [pvalue, "", ""],
+        }
+    )
+
+
 # ---#
 def explained_variance(
-    y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None
+    y_true: str, y_score: str, input_relation: Union[str, vDataFrame], cursor=None
 ):
     """
 ---------------------------------------------------------------------------
@@ -82,7 +220,7 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 
 Returns
 -------
@@ -109,7 +247,7 @@ float
 
 # ---#
 def max_error(
-    y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None
+    y_true: str, y_score: str, input_relation: Union[str, vDataFrame], cursor=None
 ):
     """
 ---------------------------------------------------------------------------
@@ -127,7 +265,7 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 
 Returns
 -------
@@ -156,7 +294,7 @@ float
 
 # ---#
 def mean_absolute_error(
-    y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None
+    y_true: str, y_score: str, input_relation: Union[str, vDataFrame], cursor=None
 ):
     """
 ---------------------------------------------------------------------------
@@ -174,7 +312,7 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 
 Returns
 -------
@@ -199,7 +337,11 @@ float
 
 # ---#
 def mean_squared_error(
-    y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None
+    y_true: str,
+    y_score: str,
+    input_relation: Union[str, vDataFrame],
+    cursor=None,
+    root: bool = False,
 ):
     """
 ---------------------------------------------------------------------------
@@ -217,7 +359,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
+root: bool, optional
+    If set to True, returns the RMSE (Root Mean Squared Error)
 
 Returns
 -------
@@ -229,12 +373,15 @@ float
             ("y_true", y_true, [str],),
             ("y_score", y_score, [str],),
             ("input_relation", input_relation, [str, vDataFrame],),
+            ("root", root, [bool],),
         ]
     )
     cursor, conn, input_relation = check_cursor(cursor, input_relation)
     query = "SELECT MSE({}, {}) OVER () FROM {}".format(y_true, y_score, input_relation)
     executeSQL(cursor, query, "Computing the MSE.")
     result = cursor.fetchone()[0]
+    if root:
+        result = math.sqrt(result)
     if conn:
         conn.close()
     return result
@@ -242,7 +389,7 @@ float
 
 # ---#
 def mean_squared_log_error(
-    y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None
+    y_true: str, y_score: str, input_relation: Union[str, vDataFrame], cursor=None
 ):
     """
 ---------------------------------------------------------------------------
@@ -260,7 +407,7 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 
 Returns
 -------
@@ -287,7 +434,7 @@ float
 
 # ---#
 def median_absolute_error(
-    y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None
+    y_true: str, y_score: str, input_relation: Union[str, vDataFrame], cursor=None
 ):
     """
 ---------------------------------------------------------------------------
@@ -305,7 +452,7 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 
 Returns
 -------
@@ -332,7 +479,7 @@ float
 
 # ---#
 def quantile_error(
-    q: float, y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None
+    q: float, y_true: str, y_score: str, input_relation: Union[str, vDataFrame], cursor=None
 ):
     """
 ---------------------------------------------------------------------------
@@ -352,7 +499,7 @@ input_relation: str/vDataFrame
     "(SELECT ... FROM ...) x" as long as an alias is given at the end of the
     relation.
 cursor: DBcursor, optional
-    Vertica DB cursor.
+    Vertica database cursor.
 
 Returns
 -------
@@ -379,7 +526,14 @@ float
 
 
 # ---#
-def r2_score(y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None):
+def r2_score(
+    y_true: str,
+    y_score: str,
+    input_relation: Union[str, vDataFrame],
+    cursor=None,
+    k: int = 0,
+    adj: bool = True,
+):
     """
 ---------------------------------------------------------------------------
 Computes the R2 Score.
@@ -396,7 +550,11 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
+k: int, optional
+    Number of predictors. Only used to compute the R2 adjusted.
+adj: bool, optional
+    If set to True, computes the R2 adjusted.
 
 Returns
 -------
@@ -408,6 +566,8 @@ float
             ("y_true", y_true, [str],),
             ("y_score", y_score, [str],),
             ("input_relation", input_relation, [str, vDataFrame],),
+            ("k", k, [int],),
+            ("adj", adj, [bool],),
         ]
     )
     cursor, conn, input_relation = check_cursor(cursor, input_relation)
@@ -416,6 +576,11 @@ float
     )
     executeSQL(cursor, query, "Computing the R2 Score.")
     result = cursor.fetchone()[0]
+    if adj and k > 0:
+        query = "SELECT COUNT(*) FROM {}".format(input_relation)
+        executeSQL(cursor, query, "Computing the table number of elements.")
+        n = cursor.fetchone()[0]
+        result = 1 - ((1 - result) * (n - 1) / (n - k - 1))
     if conn:
         conn.close()
     return result
@@ -423,7 +588,11 @@ float
 
 # ---#
 def regression_report(
-    y_true: str, y_score: str, input_relation: (str, vDataFrame), cursor=None
+    y_true: str,
+    y_score: str,
+    input_relation: Union[str, vDataFrame],
+    cursor=None,
+    k: int = 1,
 ):
     """
 ---------------------------------------------------------------------------
@@ -441,7 +610,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
+k: int, optional
+    Number of predictors. Used to compute the adjusted R2.
 
 Returns
 -------
@@ -454,6 +625,7 @@ tablesample
             ("y_true", y_true, [str],),
             ("y_score", y_score, [str],),
             ("input_relation", input_relation, [str, vDataFrame],),
+            ("k", k, [int],),
         ]
     )
     cursor, conn, input_relation = check_cursor(cursor, input_relation)
@@ -463,7 +635,9 @@ tablesample
     query += "APPROXIMATE_MEDIAN(ABS({} - {})), AVG(ABS({} - {})), ".format(
         y_true, y_score, y_true, y_score
     )
-    query += "AVG(POW({} - {}, 2)) FROM {}".format(y_true, y_score, input_relation)
+    query += "AVG(POW({} - {}, 2)), COUNT(*) FROM {}".format(
+        y_true, y_score, input_relation
+    )
     r2 = r2_score(y_true, y_score, input_relation, cursor)
     values = {
         "index": [
@@ -472,11 +646,35 @@ tablesample
             "median_absolute_error",
             "mean_absolute_error",
             "mean_squared_error",
+            "root_mean_squared_error",
             "r2",
+            "r2_adj",
+            "aic",
+            "bic",
         ]
     }
     executeSQL(cursor, query, "Computing the Regression Report.")
-    values["value"] = [item for item in cursor.fetchone()] + [r2]
+    result = cursor.fetchone()
+    n = result[5]
+    if result[4] > 0:
+        aic, bic = (
+            n * math.log(result[4]) + 2 * (k + 1) + (2 * (k + 1) ** 2 + 2 * (k + 1)) / (n - k - 2),
+            n * math.log(result[4]) + (k + 1) * math.log(n),
+        )
+    else:
+        aic, bic = -np.inf, -np.inf
+    values["value"] = [
+        result[0],
+        result[1],
+        result[2],
+        result[3],
+        result[4],
+        math.sqrt(result[4]),
+        r2,
+        1 - ((1 - r2) * (n - 1) / (n - k - 1)),
+        aic,
+        bic,
+    ]
     if conn:
         conn.close()
     return tablesample(values)
@@ -489,9 +687,9 @@ tablesample
 def accuracy_score(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
-    pos_label: (int, float, str) = 1,
+    pos_label: (int, float, str) = None,
 ):
     """
 ---------------------------------------------------------------------------
@@ -509,7 +707,7 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
 	Label to use to identify the positive class. If pos_label is NULL then the
 	global accuracy will be computed.
@@ -556,9 +754,10 @@ float
 def auc(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
+    nbins: int = 10000,
 ):
     """
 ---------------------------------------------------------------------------
@@ -576,10 +775,16 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the ROC AUC, one of the response column class has to be the 
+	To compute the ROC AUC, one of the response column classes must be the 
 	positive one. The parameter 'pos_label' represents this class.
+nbins: int, optional
+    An integer value that determines the number of decision boundaries. Decision 
+    boundaries are set at equally spaced intervals between 0 and 1, inclusive.
+    The higher it is, the more precise the AUC will be. However, it can decrease
+    considerably performances. The maximum value is 999,999. If negative, the
+    maximum value is used.
 
 Returns
 -------
@@ -595,7 +800,7 @@ float
     )
     cursor, conn, input_relation = check_cursor(cursor, input_relation)
     return roc_curve(
-        y_true, y_score, input_relation, cursor, pos_label, nbins=10000, auc_roc=True
+        y_true, y_score, input_relation, cursor, pos_label, nbins=nbins, auc_roc=True
     )
 
 
@@ -603,7 +808,7 @@ float
 def classification_report(
     y_true: str = "",
     y_score: list = [],
-    input_relation: (str, vDataFrame) = "",
+    input_relation: Union[str, vDataFrame] = "",
     cursor=None,
     labels: list = [],
     cutoff: (float, list) = [],
@@ -627,13 +832,13 @@ input_relation: str/vDataFrame, optional
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 labels: list, optional
 	List of the response column categories to use.
 cutoff: float/list, optional
 	Cutoff for which the tested category will be accepted as prediction. 
-	In case of multiclass classification, the list will represent the 
-	the classes threshold. If it is empty, the best cutoff will be used.
+	For multiclass classification, the list will represent the the classes threshold. 
+    If it is empty, the best cutoff will be used.
 estimator: object, optional
 	Estimator to use to compute the classification report.
 
@@ -702,28 +907,18 @@ tablesample
                 "DECODE({}, '{}', 1, 0)".format(y_true, elem),
             )
             matrix = confusion_matrix(y_true, y_p, input_relation, cursor, pos_label)
-        try:
-            tn, fn, fp, tp = (
-                matrix.values[non_pos_label][0],
-                matrix.values[non_pos_label][1],
-                matrix.values[pos_label][0],
-                matrix.values[pos_label][1],
-            )
-        except:
-            try:
-                tn, fn, fp, tp = (
-                    matrix.values[0][0],
-                    matrix.values[0][1],
-                    matrix.values[1][0],
-                    matrix.values[1][1],
-                )
-            except:
-                tn, fn, fp, tp = (
-                    matrix.values["0"][0],
-                    matrix.values["0"][1],
-                    matrix.values["1"][0],
-                    matrix.values["1"][1],
-                )
+        if non_pos_label in matrix.values and pos_label in matrix.values:
+            non_pos_label_, pos_label_ = non_pos_label, pos_label
+        elif 0 in matrix.values and 1 in matrix.values:
+            non_pos_label_, pos_label_ = 0, 1
+        else:
+            non_pos_label_, pos_label_ = matrix.values["index"]
+        tn, fn, fp, tp = (
+            matrix.values[non_pos_label_][0],
+            matrix.values[non_pos_label_][1],
+            matrix.values[pos_label_][0],
+            matrix.values[pos_label_][1],
+        )
         ppv = tp / (tp + fp) if (tp + fp != 0) else 0  # precision
         tpr = tp / (tp + fn) if (tp + fn != 0) else 0  # recall
         tnr = tn / (tn + fp) if (tn + fp != 0) else 0
@@ -786,7 +981,7 @@ tablesample
 def confusion_matrix(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -806,10 +1001,10 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
 	To compute the one dimension Confusion Matrix, one of the response column 
-	class has to be the positive one. The parameter 'pos_label' represents 
+	class must be the positive one. The parameter 'pos_label' represents 
 	this class.
 
 Returns
@@ -856,7 +1051,7 @@ tablesample
 def critical_success_index(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -876,9 +1071,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the CSI, one of the response column class has to be the 
+	To compute the CSI, one of the response column classes must be the 
 	positive one. The parameter 'pos_label' represents this class.
 
 Returns
@@ -912,7 +1107,7 @@ float
 def f1_score(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -932,9 +1127,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the F1 Score, one of the response column class has to be the 
+	To compute the F1 Score, one of the response column classes must be the 
 	positive one. The parameter 'pos_label' represents this class.
 
 Returns
@@ -974,7 +1169,7 @@ float
 def informedness(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -994,9 +1189,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the informedness, one of the response column class has to be the 
+	To compute the informedness, one of the response column classes must be the 
 	positive one. The parameter 'pos_label' represents this class.
 
 Returns
@@ -1031,7 +1226,7 @@ float
 def log_loss(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -1051,9 +1246,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the log loss, one of the response column class has to be the 
+	To compute the log loss, one of the response column classes must be the 
 	positive one. The parameter 'pos_label' represents this class.
 
 Returns
@@ -1082,7 +1277,7 @@ float
 def markedness(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -1102,9 +1297,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the markedness, one of the response column class has to be the 
+	To compute the markedness, one of the response column classes must be the 
 	positive one. The parameter 'pos_label' represents this class.
 
 Returns
@@ -1139,7 +1334,7 @@ float
 def matthews_corrcoef(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -1159,10 +1354,10 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
 	To compute the Matthews Correlation Coefficient, one of the response column 
-	class has to be the positive one. The parameter 'pos_label' represents this 
+	class must be the positive one. The parameter 'pos_label' represents this 
 	class.
 
 Returns
@@ -1200,7 +1395,7 @@ float
 def multilabel_confusion_matrix(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     labels: list,
     cursor=None,
 ):
@@ -1222,7 +1417,7 @@ input_relation: str/vDataFrame
 labels: list
 	List of the response column categories.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 
 Returns
 -------
@@ -1269,7 +1464,7 @@ tablesample
 def negative_predictive_score(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -1289,10 +1484,10 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
 	To compute the Negative Predictive Score, one of the response column class 
-	has to be the positive one. The parameter 'pos_label' represents this class.
+	must be the positive one. The parameter 'pos_label' represents this class.
 
 Returns
 -------
@@ -1325,30 +1520,37 @@ float
 def prc_auc(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
+    nbins: int = 10000,
 ):
     """
 ---------------------------------------------------------------------------
-Computes the PRC AUC (Area Under Curve).
+Computes the area under the curve (AUC) of a Precision-Recall (PRC) curve.
 
 Parameters
 ----------
 y_true: str
 	Response column.
 y_score: str
-	Prediction Probability.
+	Prediction probability.
 input_relation: str/vDataFrame
 	Relation to use to do the scoring. The relation can be a view or a table
 	or even a customized relation. For example, you could write:
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the PRC AUC, one of the response column class has to be the 
+	To compute the PRC AUC, one of the response column classes must be the 
 	positive one. The parameter 'pos_label' represents this class.
+nbins: int, optional
+    An integer value that determines the number of decision boundaries. Decision 
+    boundaries are set at equally-spaced intervals between 0 and 1, inclusive.
+    The greater number of decision boundaries, the greater precision, but 
+    the greater decrease in performance. Maximum value: 999,999. If negative, the
+    maximum value is used.
 
 Returns
 -------
@@ -1360,11 +1562,12 @@ float
             ("y_true", y_true, [str],),
             ("y_score", y_score, [str],),
             ("input_relation", input_relation, [str, vDataFrame],),
+            ("nbins", nbins, [int],),
         ]
     )
     cursor, conn, input_relation = check_cursor(cursor, input_relation)
     return prc_curve(
-        y_true, y_score, input_relation, cursor, pos_label, nbins=10000, auc_prc=True
+        y_true, y_score, input_relation, cursor, pos_label, nbins=nbins, auc_prc=True,
     )
 
 
@@ -1372,7 +1575,7 @@ float
 def precision_score(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -1392,9 +1595,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the Precision Score, one of the response column class has to be 
+	To compute the Precision Score, one of the response column classes must be 
 	the positive one. The parameter 'pos_label' represents this class.
 
 Returns
@@ -1428,7 +1631,7 @@ float
 def recall_score(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -1448,9 +1651,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the Recall Score, one of the response column class has to be 
+	To compute the Recall Score, one of the response column classes must be 
 	the positive one. The parameter 'pos_label' represents this class.
 
 Returns
@@ -1484,7 +1687,7 @@ float
 def specificity_score(
     y_true: str,
     y_score: str,
-    input_relation: (str, vDataFrame),
+    input_relation: Union[str, vDataFrame],
     cursor=None,
     pos_label: (int, float, str) = 1,
 ):
@@ -1504,9 +1707,9 @@ input_relation: str/vDataFrame
 	"(SELECT ... FROM ...) x" as long as an alias is given at the end of the
 	relation.
 cursor: DBcursor, optional
-	Vertica DB cursor.
+	Vertica database cursor.
 pos_label: int/float/str, optional
-	To compute the Specificity Score, one of the response column class has to 
+	To compute the Specificity Score, one of the response column classes must 
 	be the positive one. The parameter 'pos_label' represents this class.
 
 Returns
