@@ -1,4 +1,4 @@
-# (c) Copyright [2018-2020] Micro Focus or one of its affiliates.
+# (c) Copyright [2018-2021] Micro Focus or one of its affiliates.
 # Licensed under the Apache License, Version 2.0 (the "License");
 # You may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -56,8 +56,12 @@
 ##
 #
 # ---#
-def sql(line, cell=""):
-    from verticapy.connections.connect import read_auto_connect
+from IPython.core.magic import needs_local_scope
+
+@needs_local_scope
+def sql(line, cell="", local_ns=None):
+    import verticapy
+    from verticapy.toolbox import optimized_conn
     from verticapy.utilities import readSQL
     from verticapy.utilities import vdf_from_relation
     from IPython.core.display import HTML, display
@@ -68,8 +72,7 @@ def sql(line, cell=""):
 
     version = vertica_python.__version__.split(".")
     version = [int(elem) for elem in version]
-    conn = read_auto_connect()
-    cursor = conn.cursor()
+    conn, cursor = optimized_conn()
     queries = line if (not (cell) and (line)) else cell
     options = {"limit": 100, "vdf": False}
     queries = queries.replace("\t", " ")
@@ -91,7 +94,7 @@ def sql(line, cell=""):
                 options["limit"] = int(all_options_dict[option])
             elif option.lower() == "-vdf":
                 options["vdf"] = bool(all_options_dict[option])
-            else:
+            elif verticapy.options["print_info"]:
                 print(
                     "\u26A0 Warning : The option '{}' doesn't exist, it was skipped.".format(
                         option
@@ -143,6 +146,8 @@ def sql(line, cell=""):
             if (query.split(" ")[0])
             else query.split(" ")[1].upper()
         )
+        if len(query_type) > 1 and query_type[0:2] in ("/*", "--"):
+            query_type = "undefined"
         if (
             (query_type == "COPY")
             and ("from local" in query.lower())
@@ -164,9 +169,10 @@ def sql(line, cell=""):
                 file_name = file_name[1:-1]
             with open(file_name, "r") as fs:
                 cursor.copy(query, fs)
-        elif (i < n - 1) or ((i == n - 1) and (query_type.lower() != "select")):
+        elif (i < n - 1) or ((i == n - 1) and (query_type.lower() not in ("select", "with", "undefined"))):
             cursor.execute(query)
-            print(query_type)
+            if verticapy.options["print_info"]:
+                print(query_type)
         else:
             error = ""
             try:
@@ -175,22 +181,20 @@ def sql(line, cell=""):
                 else:
                     result = readSQL(query, cursor=cursor, limit=options["limit"],)
             except:
-                raise
                 try:
                     cursor.execute(query)
                     final_result = cursor.fetchone()
-                    if final_result:
+                    if final_result and verticapy.options["print_info"]:
                         print(final_result[0])
-                    else:
+                    elif verticapy.options["print_info"]:
                         print(query_type)
                 except Exception as e:
                     error = e
             if error:
                 raise QueryError(error)
-    if not (options["vdf"]):
-        conn.close()
     elapsed_time = time.time() - start_time
-    display(HTML("<div><b>Execution: </b> {}s</div>".format(round(elapsed_time, 3))))
+    if verticapy.options["print_info"]:
+        display(HTML("<div><b>Execution: </b> {}s</div>".format(round(elapsed_time, 3))))
     return result
 
 
