@@ -1,4 +1,4 @@
-# (c) Copyright [2018-2020] Micro Focus or one of its affiliates.
+# (c) Copyright [2018-2021] Micro Focus or one of its affiliates.
 # Licensed under the Apache License, Version 2.0 (the "License");
 # You may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -12,41 +12,39 @@
 # limitations under the License.
 
 import pytest, warnings
-from verticapy import vDataFrame, drop_table, errors
-
-from verticapy import set_option
+from verticapy import vDataFrame, drop, errors, set_option, tablesample
 
 set_option("print_info", False)
 
 
 @pytest.fixture(scope="module")
 def titanic_vd(base):
-    from verticapy.learn.datasets import load_titanic
+    from verticapy.datasets import load_titanic
 
     titanic = load_titanic(cursor=base.cursor)
     yield titanic
     with warnings.catch_warnings(record=True) as w:
-        drop_table(name="public.titanic", cursor=base.cursor)
+        drop(name="public.titanic", cursor=base.cursor)
 
 
 @pytest.fixture(scope="module")
 def iris_vd(base):
-    from verticapy.learn.datasets import load_iris
+    from verticapy.datasets import load_iris
 
     iris = load_iris(cursor=base.cursor)
     yield iris
     with warnings.catch_warnings(record=True) as w:
-        drop_table(name="public.iris", cursor=base.cursor)
+        drop(name="public.iris", cursor=base.cursor)
 
 
 @pytest.fixture(scope="module")
 def market_vd(base):
-    from verticapy.learn.datasets import load_market
+    from verticapy.datasets import load_market
 
     market = load_market(cursor=base.cursor)
     yield market
     with warnings.catch_warnings(record=True) as w:
-        drop_table(name="public.market", cursor=base.cursor)
+        drop(name="public.market", cursor=base.cursor)
 
 
 class TestvDFPreprocessing:
@@ -57,6 +55,27 @@ class TestvDFPreprocessing:
         assert train.shape() == (pytest.approx(809), 14)
         assert test.shape() == (pytest.approx(425), 14)
 
+    def test_vDF_add_duplicates(self, base):
+        names = tablesample({"name": ["Badr", "Waqas", "Pratibha"], "weight": [2, 4, 6]}).to_vdf(cursor=base.cursor)
+        result = names.add_duplicates("weight").groupby("name", "COUNT(*) AS cnt").sort("cnt")
+        assert result[0] == ['Badr', 1]
+        assert result[1] == ['Waqas', 2]
+        assert result[2] == ['Pratibha', 3]
+
+    def test_vDF_cdt(self, titanic_vd):
+        result = titanic_vd[["age", "fare", "pclass", "boat"]].cdt()
+        assert result.shape() == (1234, 41)
+        assert result["boat_11"].avg() == pytest.approx(-0.999189627228525)
+
+    def test_vDF_cut(self, titanic_vd):
+        titanic_copy = titanic_vd.copy()
+        titanic_copy["age"].cut([0, 15, 80])
+        assert sorted(titanic_copy["age"].distinct()) == ['[0;15]', ']15;80]']
+        titanic_copy["fare"].cut([0, 15, 800], right=False, include_lowest=False,)
+        assert sorted(titanic_copy["fare"].distinct()) == ['[15;800[', ']0;15[']
+        titanic_copy["parch"].cut([0, 5, 10], right=False, include_lowest=False, labels=["small", "big"],)
+        assert sorted(titanic_copy["parch"].distinct()) == ['big', 'small']
+
     def test_vDF_decode(self, titanic_vd):
         titanic_copy = titanic_vd.copy()
         titanic_copy["sex"].decode("female", 1, "male", 0, 2)
@@ -83,7 +102,7 @@ class TestvDFPreprocessing:
         titanic_copy = titanic_vd.copy()
 
         # expected exception
-        with pytest.raises(errors.ParameterError) as exception_info:
+        with pytest.raises(AssertionError) as exception_info:
             titanic_copy["age"].discretize(method="same_freq", bins=1)
         # checking the error message
         assert exception_info.match(
@@ -96,7 +115,8 @@ class TestvDFPreprocessing:
             "[0.330;21.000]",
             "[21.000;28.000]",
             "[28.000;39.000]",
-            "[39.000;80.000]",
+            "[39.000;76.000]",
+            "[76.000;80.000]",
         ]
 
         ### method = "smart"
@@ -116,7 +136,7 @@ class TestvDFPreprocessing:
         titanic_copy["name"].str_extract(" ([A-Za-z])+\\.")
 
         # expected exception
-        with pytest.raises(errors.ParameterError) as exception_info:
+        with pytest.raises(AssertionError) as exception_info:
             titanic_copy["name"].discretize(method="topk", k=0, new_category="rare")
         # checking the error message
         assert exception_info.match(
@@ -124,13 +144,15 @@ class TestvDFPreprocessing:
             "discretization using the method 'topk'"
         )
 
-        titanic_copy["name"].discretize(method="topk", k=5, new_category="rare")
-        assert titanic_copy["name"].distinct() == [
+        titanic_copy["name"].discretize(method="topk", k=6, new_category="rare")
+        result = titanic_copy["name"].distinct()
+        assert result == [
             " Dr.",
             " Master.",
             " Miss.",
             " Mr.",
             " Mrs.",
+            " Rev.",
             "rare",
         ]
 
@@ -279,11 +301,11 @@ class TestvDFPreprocessing:
         assert titanic_copy["fare"].count() == 1234
 
         assert titanic_copy["cabin"].count() == 286
-        titanic_copy["cabin"].fillna(method="bfill", order_by=["pclass", "cabin"])
+        titanic_copy["cabin"].fillna(method="ffill", order_by=["pclass", "cabin"])
         assert titanic_copy["cabin"].count() == 1234
 
         assert titanic_copy["home.dest"].count() == 706
-        titanic_copy["home.dest"].fillna(method="ffill", order_by=["ticket"])
+        titanic_copy["home.dest"].fillna(method="bfill", order_by=["ticket"])
         assert titanic_copy["home.dest"].count() == 1234
 
         assert titanic_copy["body"].count() == 118
@@ -397,7 +419,7 @@ class TestvDFPreprocessing:
         with pytest.raises(errors.ConversionError) as exception_info:
             titanic_copy["sex"].astype("int")
         # checking the error message
-        assert exception_info.match('The vcolumn "sex" can not be converted to int')
+        assert exception_info.match('Could not convert "female" from column titanic.sex to an int8')
 
         titanic_copy["sex"].astype("varchar(10)")
         assert titanic_copy["sex"].dtype() == "varchar(10)"
